package br.com.previna.bo;

import br.com.previna.dao.FaseDAO;
import br.com.previna.dao.HistoriaDAO;
import br.com.previna.dao.NodoDAO;
import br.com.previna.dto.FaseFormattedDTO;
import br.com.previna.exception.ValidationException;
import br.com.previna.model.*;

import java.util.ArrayList;
import java.util.List;

public class FaseBO {

	FaseDAO faseDAO = new FaseDAO();
	NodoDAO nodoDAO = new NodoDAO();
	HistoriaDAO historiaDAO = new HistoriaDAO();
	PerguntaBO perguntaBO = new PerguntaBO();
	ConteudoBO conteudoBO = new ConteudoBO();
	HistoriaBO historiaBO = new HistoriaBO();
	NodoBO nodoBO = new NodoBO();

	public boolean add(Fase fase) throws ValidationException, Exception {
		try {
			validate(fase);
			criaNodos(fase);
			faseDAO.save(fase);
			return true;
		} catch (ValidationException e) {
			throw e;
		} catch (Exception e) {
			throw new Exception("Erro ao adicionar a Fase: " + e.getMessage());
		}
	}

	public boolean criaNodos(Fase fase) throws Exception {
		for (int i = 0; i < fase.getConexoes().size(); i++) {
			Nodo n = fase.getConexoes().get(i).getNodoInicial();
			fase.getConexoes().get(i).setNodoInicial(nodoBO.nodoExiste(n));

			fase.getConexoes().get(i).setNodoCerto(nodoBO.nodoExiste(fase.getConexoes().get(i).getNodoCerto()));
			if (fase.getConexoes().get(i).getNodoErrado() != null)
				fase.getConexoes().get(i).setNodoErrado(nodoBO.nodoExiste(fase.getConexoes().get(i).getNodoErrado()));
		}
		return true;
	}

	public void finalExiste() {
		try {
			if (nodoDAO.findId(1, Nodo.class) == null) {
				Nodo nodo = new Nodo();
				Historia historia = new Historia();
				historia.setDescricao("Final da Fase , parabéns");
				nodo.setHistoria(historia);
				nodoDAO.save(nodo);
			}
		} catch (Exception e) {

		}
	}
	
	private void validate(Fase fase) throws ValidationException {
		if(fase.getConexoes().size() < 2)
			throw new ValidationException("Número de conexões abaixo do mínimo permitido");
		
		if(fase.getAgeGroup() == null || fase.getAgeGroup().getId() == 0)
			throw new ValidationException("Faixa etária inválida");
		
		try {
			@SuppressWarnings("unused")
			int dificuldade = fase.getDificuldade();
		} catch (NullPointerException e) {
			throw new ValidationException("Dificuldade não especificada");
		}
		
	}

	public List<Fase> list() {
		try {
			return faseDAO.listAll(Fase.class);
		} catch (Exception e) {
			return null;

		}
	}

	public List<FaseFormattedDTO> listFormatted() {
		try {
			List<FaseFormattedDTO> faseFormattedDTOList = new ArrayList<>();

			for (Fase f : faseDAO.listAll(Fase.class)) {
				faseFormattedDTOList.add(new FaseFormattedDTO(f));
			}
			return faseFormattedDTOList;
		} catch (Exception e) {
			return null;
		}
	}

	public Fase findId(Long id) {
		return faseDAO.findId(id, Fase.class);
	}
	
	public void remove(Fase fase) throws Exception {
		faseDAO.removeFase(fase);
	}
}
