package br.com.previna.controllers;

import br.com.previna.bo.FaseBO;
import br.com.previna.dto.FaseFormattedDTO;
import br.com.previna.exception.PersistenciaException;
import br.com.previna.exception.ValidationException;
import br.com.previna.model.Fase;

import javax.persistence.EntityNotFoundException;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import java.util.List;

@Path("fase")
public class FaseController {
FaseBO faseBO = new FaseBO();

    @Context
    private HttpServletRequest request;

    @POST
    @Path("/criarFase")
    @Consumes("application/json; charset=UTF-8")
    @Produces("application/json; charset=UTF-8")
    @JWTTokenNeeded
    public Response create(Fase fase) {
        try {
            faseBO.add(fase);
            return Response.ok().entity(faseBO.list()).status(Response.Status.ACCEPTED).build();
        } catch (Exception e){
            return Response.ok().entity(e.getMessage()).status(Response.Status.BAD_REQUEST).build();
        }

    }

    @GET
    @Path("/listarFase")
    @Produces("application/json; charset=UTF-8")
    @JWTTokenNeeded
    public Response list(){
        try {
            List<Fase> faseList = faseBO.list();
            return Response.ok().entity(faseList).build();
        }catch (Exception e){
            return Response.ok().entity(e.getMessage()).status(Response.Status.BAD_REQUEST).build();
        }
    }

    @GET
    @Path("/listarFaseMenu")
    @Produces("application/json; charset=UTF-8")
    @JWTTokenNeeded
    public Response listMenu(){
        try {
            List<FaseFormattedDTO> faseFormattedDTOList = faseBO.listFormatted();
            return Response.ok().entity(faseFormattedDTOList).build();
        }catch (Exception e){
            return Response.ok().entity(e.getMessage()).status(Response.Status.BAD_REQUEST).build();
        }
    }

    @GET
    @Path("/buscarFase/{id}")
    @Produces("application/json; charset=UTF-8")
    @JWTTokenNeeded
    public Response searchByID(@PathParam("id") Long id) throws PersistenciaException, ValidationException {
        try {
            return Response.ok().entity(faseBO.findId(id)).status(Response.Status.ACCEPTED).build();
        } catch (Exception e) {
            return Response.ok().entity(e.getMessage()).status(Response.Status.BAD_REQUEST).build();
        }
    }
    
    @DELETE
    @Path("/removerFase/{id}")
    @Produces("application/json; charset=UTF-8")
    @JWTTokenNeeded
    public Response remove(@PathParam("id") Long id) throws PersistenciaException, ValidationException {
        try {
            Fase fase = faseBO.findId(id);
            if(fase == null)
            	throw new EntityNotFoundException("Fase não encontrada");
            faseBO.remove(fase);
        } catch (Exception e) {
            return Response.ok().entity(e.getMessage()).status(Response.Status.BAD_REQUEST).build();
        }
        return Response.ok().status(Response.Status.ACCEPTED).build();
    }
}
